/* ----------------------------------------
 * Origin  : GFI INFORMATIQUE 
 * Project : CDPP Propagation Tool 
 * ----------------------------------------
 * History 
 * Version : 01/05/2015 : SCA : V2.1
 * Version : 06/11/2017 : SCA : V2.2.1 
 *  Redmine #5403 - DMPT3 
 * Version : 30/11/2022 : LBE : V2.6
 * DM-ID : Redmine #l0481 : 30/11/2022 : HCS and PFSS FL added to carrington map
 * Version : 05/08/2024 : LBE : V2.11
 * DM-ID : Redmine#l0763 : 05/08/2024 : Field lines menu replaced by cbx
 * ----------------------------------------
 */
package fr.tlse.irap.pt.client.controller.tools.carrington;

import fr.tlse.irap.pt.client.controller.common.ISpaceElementMenu;
import fr.tlse.irap.flare.Flare.FlareType;
import fr.tlse.irap.map.CarringtonMapType;

public interface ICarringtonMenu extends ISpaceElementMenu {

	public void runCarringtonMap(CarringtonMapType cmType);
	public void runCMEClickSource();
	public void runCMEClickExtent();
	public void runShowAllFlares();
	public void runShowFlares(FlareType ft);
	public void runShowShocks();
	public void runShowHcs();
	public void runReset();
}
